"""
Simple tool for checking a bunch of Nextcloud servers, whether their Nextcloud version is supported and up2date.

 Author: wh0ami
License: MIT License <https://opensource.org/license/MIT>
Project: https://codeberg.org/wh0ami/NextcloudVersionChecker
"""

import urllib.error

from loguru import logger

from nextcloud_version_checker.nextcloud.instance import fetch_current_nextcloud_instance_version
from nextcloud_version_checker.nextcloud.updater_server import fetch_major_version_information
from nextcloud_version_checker.nextcloud.versioning import semver_to_nextcloud_version


def red(string: str) -> str:
    """Color a string bold red."""
    return f"\033[1m\033[91m{string}\033[0m"


def green(string: str) -> str:
    """Color a string bold green."""
    return f"\033[1m\033[92m{string}\033[0m"


def yellow(string: str) -> str:
    """Color a string bold orange."""
    return f"\033[1m\033[93m{string}\033[0m"


def check(instance_url: str) -> list[str]:
    """
    Check the current update status of a Nextcloud instance.

    :param instance_url: URL of the Nextcloud instance to check.
    :return: Tabulate compatible list with data about the current status of the Nextcloud instance.
    """
    try:
        current_version = fetch_current_nextcloud_instance_version(url=instance_url)
    except urllib.error.URLError as exception:
        logger.error(f"Error while fetching the current version of '{instance_url}':")
        logger.error(f"-> {exception}")
        current_version = None

    if current_version:
        try:
            major_version_information = fetch_major_version_information(major_version=current_version.major)
        except urllib.error.URLError as exception:
            logger.error(f"Error while fetching the current version of '{instance_url}':")
            logger.error(f"-> {exception}")
            major_version_information = None

        if major_version_information:
            current_version_string = semver_to_nextcloud_version(version=current_version)
            supported = red("✖") if major_version_information["is_eol"] else green("✓")
            latest = red("✖") if major_version_information["latest_version"] > current_version else green("✓")
            available_version = (
                yellow(
                    semver_to_nextcloud_version(
                        version=major_version_information["latest_version"],
                    ),
                )
                if major_version_information["latest_version"] > current_version
                else "-"
            )
    else:
        current_version_string = red("Error")
        supported = "?"
        latest = "?"
        available_version = "?"

    return [
        instance_url.split("://")[1],
        current_version_string,
        supported,
        latest,
        available_version,
    ]
